﻿Partial Class dtsTree

	Public Shared ReadOnly dts_Tree As dtsTree

	Shared Sub New()
		dts_Tree = New dtsTree
	End Sub

	Public Sub ReloadTree()
		Me.Tree.Clear()
		Me.Tree.ReadXml(DataFile.FullName)
	End Sub
	Public Sub SaveTree()
		Tree.WriteXml(DataFile.FullName)
		Tree.AcceptChanges()
	End Sub

	Public Sub FillTree(tv As TreeView)
		tv.Nodes.Clear()
		Dim mode As String = My.Settings.Mitarbeiter_Funktion
		Dim filteredSorted = Tree.Select($"{mode} = True").Cast(Of TreeRow).OrderBy(Function(rw) rw.SortKey).ToList
		Dim nodeCollections = {tv.Nodes, Nothing, Nothing, Nothing}
		For Each rw In filteredSorted
			Dim iNodes = rw.Nr.Split("."c).Length - 1
			nodeCollections(iNodes + 1) = nodeCollections(iNodes).Add($"{rw.Key}", $"{rw.Nr} = {rw.Bezeichnung}").Nodes
		Next
		tv.ExpandAll()
	End Sub

	''' <summary>erzeugt aus einem hierarchischen InhaltsVerzeichnis-Eintrag (TableOfContent - TOC) einen sich lexikografisch korrekt einordnenden SortKey.
	'''  Beispiel: "2.11.3 Blabla" konvertiert zu " 2 B 3" (verkettete 2-stellig-rechtsbündige Hex-Darstellung der Segmente)
	'''  MaxValue eines tocEntry-Segments ist folglich die grösste 2-stellige hex-darstellung: FF, in decimal: 255</summary>
	Private Shared Function GetTocSortKey(tocEntry As String) As String
		Dim spaceIndex = tocEntry.IndexOf(" "c)
		Select Case spaceIndex
			Case 0 : Throw New ArgumentException("mustn't start with ' '", NameOf(tocEntry))
			Case > 0 : tocEntry = tocEntry.Substring(0, spaceIndex)
		End Select
		Dim numbs = tocEntry.Split("."c).Select(AddressOf Integer.Parse).ToArray
		If numbs.Any(Function(n) n > Byte.MaxValue) Then Throw New ArgumentException("segments maximum is '255'", NameOf(tocEntry))
		Return String.Concat(numbs.Select(Function(n) $"{n,2:X}")) ' 2-stellig rechtsbündige Hex-Darstellung
	End Function

	Partial Class TreeDataTable
		Private Sub TreeDataTable_ColumnChanged(sender As Object, e As DataColumnChangeEventArgs) Handles Me.ColumnChanged
			If e.Column IsNot NrColumn Then Return
			Dim rw = DirectCast(e.Row, TreeRow)
			rw.SortKey = GetTocSortKey(rw.Nr)
		End Sub
	End Class 'TreeDataTable

	Partial Class TreeRow
		Public Overrides Function ToString() As String ' Debugging-Hilfe: Im Haltemodus zeigt der Debugger nun die signifikante Eigenschaft der TreeRow
			Return Nr
		End Function
	End Class 'TreeRow

End Class 'dtsTree